@@ -75,6 +75,20 @@ gnome_volume_control_preferences_class_init (GnomeVolumeControlPreferencesClass
   gtkdialog_class->response = gnome_volume_control_preferences_response;
 }
 
+/*
+ * Mixer tracks are sorted by their types.
+ */
+static gint
+sort_by_page_num (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer user_data)
+{
+	GstMixerTrack *a_track, *b_track;
+
+	gtk_tree_model_get (model, a, COL_TRACK, &a_track, -1);
+	gtk_tree_model_get (model, b, COL_TRACK, &b_track, -1);
+
+	return get_page_num (a_track) - get_page_num (b_track);
+}
+
 static void
 gnome_volume_control_preferences_init (GnomeVolumeControlPreferences *prefs)
 {
@@ -109,6 +123,8 @@ gnome_volume_control_preferences_init (GnomeVolumeControlPreferences *prefs)
 
   store = gtk_list_store_new (NUM_COLS, G_TYPE_BOOLEAN,
 			      G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_STRING);
+  gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (store), sort_by_page_num, NULL, NULL);
+  gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store), GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
   prefs->treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
   gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (prefs->treeview), FALSE);
   gtk_label_set_mnemonic_widget (GTK_LABEL(label), GTK_WIDGET (prefs->treeview));
@@ -239,13 +255,10 @@ get_gconf_key (GstMixer *mixer, GstMixerTrack *track)
   dev = g_object_get_data (G_OBJECT (mixer),
 			   "gnome-volume-control-name");
   if (track != NULL) {
-    if (g_object_class_find_property (G_OBJECT_GET_CLASS (track), "untranslated-label"))
-      g_object_get (track, "untranslated-label", &label, NULL);
-    if (label == NULL)
-      g_object_get (track, "label", &label, NULL);
-  }
-  if (label == NULL)
+    label = g_strdup (track->label);
+  } else {
     label = g_strdup ("");
+  }
 
   pos = strlen (GNOME_VOLUME_CONTROL_KEY_DIR) + 1;
   res = g_new (gchar, pos + strlen (dev) + 1 + strlen (label) + 1);
@@ -288,7 +301,6 @@ gnome_volume_control_preferences_change (GnomeVolumeControlPreferences *prefs,
   /* remove old */
   while (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (store), &iter)) {
     gtk_list_store_remove (store, &iter);
-	/* does this free COL_TYPE? */
   }
 
   /* take/put reference */
@@ -313,6 +325,7 @@ gnome_volume_control_preferences_change (GnomeVolumeControlPreferences *prefs,
 			COL_ACTIVE, active,
 			COL_LABEL, track->label,
 			COL_TRACK, track,
+			COL_TYPE, get_page_description (get_page_num (track)),
 			-1);
   }
 }
@@ -326,13 +339,9 @@ set_gconf_track_active(GConfClient *client, GstMixer *mixer,
 		       GstMixerTrack *track, gboolean active)
 {
   gchar *key;
-  GConfValue *value;
 
   key = get_gconf_key (mixer, track);
-  value = gconf_value_new (GCONF_VALUE_BOOL);
-  gconf_value_set_bool (value, active);
-  gconf_client_set (client, key, value, NULL);
-  gconf_value_free (value);
+  gconf_client_set_bool (client, key, active, NULL);
   g_free (key);
 }
 
